"""Calculates contourlines from 2d image data

Because the calculation of contours in python is quite slow, the function
contourcache will cache contourlines on disk. Changed in the data are
detected my the use of a (md5) checksum.

example:
	import Kplot
	data2d = [[0,0.5,1,0.5,0], [0,1,2,1,0], [0,0.5,1,0.5,0]]
	x, y = Kplot.algo.contour.lines(data2d, 1.3)

cached example
	import Kplot
	data2d = [[0,0.5,1,0.5,0], [0,1,2,1,0], [0,0.5,1,0.5,0]]
	x, y = Kplot.algo.contour.cachedlines("contour.cache", data2d, 1.3)

The x and y array can be used with the Kplot.Lines PlotObject
example:
	import Kplot
	data2d = [[0,1,0], [1,2,1], [0,1,0]]
	x, y = Kplot.algo.contour.cachedlines("contour.cache", data2d, 1.3)

	image = Kplot.Image(data2d, colormap=Kplot.ColorMap.blackwhite)
	lines = Kplot.Lines(x,y, color="red")

	dev = Kplot.createDevice()
	panel = dev.createPanel()
	panel.setWorldImage(data2d, margin=0.5)
	panel.add(image, lines)
	dev.output()
	dev.close()


"""

import Numeric
from Numeric import *
from Kplot.plotobjects.properties import *
from Kplot.plotobjects.plotobject import PlotObject
import Kplot
import os

def _fillRow(data2d, row, rowNr, level):
	next = data2d[rowNr][0]
	curr = next
	for x in range(len(row)-1):
		curr = next
		next = data2d[rowNr][x+1]
		if(not((curr <  level and next < level) or (curr >= level and next >= level)) ):
			if(abs(next - curr) > 0.0001):
				row[x] = x + (level - curr) / (next - curr)
			else:
				row[x] = x
		else:
				row[x] = -1
	return

def __fillRow(data2d, row, rowNr, level):
	curr = data2d[rowNr][:-1]
	next = data2d[rowNr][1:]
	WIDTH = len(row)
	val = arange(WIDTH) + (-curr + level) / (next - curr)
	Numeric.put(row, arange(WIDTH), val)
	mask = ((curr <  level) & (next < level)) | ((curr >= level) & (next >= level))
	Numeric.putmask(row, mask, [-1])


def lines(data2d, level):
	"""Calculates the lines that form the contour

	Return an x, and y array of even length.
	Each line starts at x[n], y[n], and ends in x[n+1], y[n+1](n is even).
	
	Arguments:
	data2d -- 2dimensional sequence
	level

	"""
	totallen = 0
	HEIGHT = len(data2d)
	WIDTH = len(data2d[0])
	row1 = [0.0] * WIDTH #Numeric.zeros(WIDTH)
	row2 = [0.0] * WIDTH #Numeric.zeros(WIDTH)
	#row1 = Numeric.zeros(WIDTH-1, Float)
	#row2 = Numeric.zeros(WIDTH-1, Float)
	rowbottom = row1
	rowtop = row2
	_fillRow(data2d, rowtop, 0, level)
	#res = Numeric.zeros((HEIGHT-1)*(WIDTH-1))
	#res =
	def drawLine(x1, y1, x2, y2, totallen, xar, yar):
		#xar[totallen] = x1+0.5;
		#yar[totallen] = y1+0.5;
		#xar[totallen+1] = x2+0.5;
		#yar[totallen+1] = y2+0.5;
		xar[totallen] = x1+1.0;
		yar[totallen] = y1+1.0;
		xar[totallen+1] = x2+1.0;
		yar[totallen+1] = y2+1.0;
		return totallen+2
	xar = Numeric.zeros(WIDTH*HEIGHT*4, Float)
	yar = Numeric.zeros(WIDTH*HEIGHT*4, Float)
	for y in xrange(HEIGHT-1):
		rowbottom, rowtop = rowtop, rowbottom
		_fillRow(data2d, rowtop, y+1, level)
		y1 = data2d[y][0]
		y2 = data2d[y+1][0]

		if(not((y1 <  level and y2 < level) or (y1 >= level and y2 >= level)) ):
			next = y + (level - y1) / (y2 - y1)
		else:
			next = -1

		for x in xrange(WIDTH-1):
			top = rowtop[x]
			bottom = rowbottom[x]
			curr = next
			y1 = data2d[y][x+1]
			y2 = data2d[y+1][x+1]
			if(not((y1 <  level and y2 < level) or (y1 >= level and y2 >= level)) ):
				next = y + (level - y1) / (y2 - y1)
			else:
				next = -1
			if curr != -1:
				if(bottom != -1):
					totallen = drawLine(x, curr, bottom, y, totallen, xar, yar)
				elif(next != -1):
					totallen = drawLine(x, curr, x+1, next, totallen, xar, yar)
				elif(top != -1):
					totallen = drawLine(x, curr, top, y+1, totallen, xar, yar)
			elif(bottom != -1):
				if(next != -1):
					totallen = drawLine(bottom, y, x+1, next, totallen, xar, yar)
				elif(top != -1):
					totallen = drawLine(bottom, y, top, y+1, totallen, xar, yar)
			elif(next != -1):
				if(top != -1):
					totallen = drawLine(x+1, next, top, y+1, totallen, xar, yar)
	#totallen = totallen - 2
	return (xar[:totallen], yar[:totallen])


#def part(ar, p1, p2):
#	new = ar[p1[0]:p1[1]]
#	new = transpose(new)
#	new = new[p2[0]:p2[1]]
#	new = transpose(new)
#	return new

import pickle
import md5


def cachedlines(cachefile, data2d, level, debug=True):
	"""Caches the contourline in the cachefile

	The (md5) checksum the level and the contourlines are stored
	in the cachefile. When the data has changed, or the level has changed,
	the contour is recalculated.

	Arguments:
	cachefile -- filename of the cachefile
	data2d -- the originaldata on which the contourlines are based
	level -- the level or height of the contourlines
	debug -- if True, prints out debug info
	"""
	data2d = array(data2d)
	def newContour(md5sig):
		x, y = lines(data2d, level)
		f = file(cachefile, "wb")
		pickle.dump((md5sig, level, x, y), f, 1)
		f.close()
		return (x, y)

	md5sig = md5.md5(data2d).digest()
	nocachefile = False
	try:
		cachedate = os.stat(cachefile)[-1]
	except:
		nocachefile = True
	if nocachefile:
		if debug:
			print "cachefile %s found, generating new contourlines" % cachefile
		return newContour(md5sig)
	else:
		f = file(cachefile, "rb")
		try:
			orgmd5sig, orglevel, x, y = pickle.load(f)
		except:
			print "error loading cache file(%s), generating new contourlines" % cachefile
			return newContour(md5sig)
		if (md5sig != orgmd5sig):
			print "md5 signature doesn't match(data2d has changed), generating new contourlines"
			return newContour(md5sig)
		elif (level != orglevel):
			print "contourlevel has changed, generating new contourlines"
			return newContour(md5sig)
		else:
			print "succesfull cache, using cached contourlines(file: %s)" % (cachefile)
			return (x, y)

try:
	#import psyco
	#psyco.bind(_fillRow)
	#psyco.bind(contourlines)
	pass
except:
	pass

import Kplot
import sys

def demo():
	width = 40
	height = 40
	data2d = zeros((width,height),Float32)
	for i in range(1,width+1):
		for j in range(1,height+1):
			data2d[i-1,j-1] = cos(.3*sqrt(2*i) - .4*j/3)*cos(.4*i/3) + (i-j)/40.0
		
	dataflat = data2d.flat
	datamin, datamax = min(dataflat), max(dataflat)

	image = Kplot.Image(data2d, colormap=Kplot.ColorMap.blackwhite)
	box = Kplot.Box()

	dev = Kplot.createDevice()
	panel = dev.createPanel()
	panel.setWorldImage(data2d, margin=0.0)
	panel.add(image, box)
	for i in range(10):
		c = datamin + i*((datamax - datamin) / (10-1))
		x, y = Kplot.algo.contour.cachedlines("contour.cache.%i" % i, data2d, c)
		lines = Kplot.Lines(x,y, color="red")
		panel.add(lines)
	dev.output()
	dev.close()

if __name__ == "__main__":
	#data = FITS(sys.argv[1]).data
	#im = Kplot.simple.Image(data, colormap=Kplot.ColorMap.cool)
	#level = min(data.flat) + (max(data.flat) - min(data.flat)) / 2.0
	#data2d = array([[1,2,4,2,3,2],[4,2,1,3,2,1],[2,3,1,2,2,5],[4,2,1,3,2,1]])
	#width = 140
	#height = 140
	#surf = zeros((width,height),Float32)
	#for i in range(1,width+1):
	#	for j in range(1,height+1):
	#		surf[i-1,j-1] = cos(.3*sqrt(2*i) - .4*j/3)*cos(.4*i/3) + (i-j)/40.0

	data2d = [[0,1,0], [1,2,1], [0,1,0]]
	x, y = Kplot.algo.contour.cachedlines("contour.cache", data2d, 1.3)

	image = Kplot.Image(data2d, colormap=Kplot.ColorMap.blackwhite)
	lines = Kplot.Lines(x,y, color="red")

	dev = Kplot.createDevice()
	panel = dev.createPanel()
	panel.setWorldImage(data2d, margin=0.5)
	panel.add(image, lines)
	dev.output()
	dev.close()
