"""An r,g,b Color class and predifined colors"""
import math
import types
import Kplot
import Numeric

class Color(object):
	"""
	Colors consist of values red, green and blue values between 0 and 1

	3 ways to get a color

	redcolor = Color(1,0,0)
	redcolor = Color.red
	redcolor = Color.fromString("red")

	the Kplot.fromString method is the reason that in Kplot you may use a string
	instead of a Kplot.Color object to represent a color.
	"""
	def __init__(self, r, g, b):
		"""Constructor

		Arguments:
		r,g,b -- A value between 0 and 1

		"""
		self.r = float(r)
		self.g = float(g)
		self.b = float(b)

	def __eq__(self, color):
		if color == None:
			return 0
		elif isinstance(color, Color):
			return self.r == color.r and self.g == color.g and self.b == color.b
		else:
			return 0

	def __ne__(self, color):
		return not self.__eq__(color)

	def __repr__(self):
		return self.__str__()

	def __str__(self):
		#return "<class "+self.__module__ +"." +self.__class__.__name__+" instance at " +hex(self.__hash__()) +" r=%1.4f g=%1.4f b=%1.4f> " % (self.r, self.g, self.b)# python2.2
		return "<class "+self.__module__ +"." +self.__class__.__name__+" instance r=%1.4f g=%1.4f b=%1.4f> " % (self.r, self.g, self.b)# python 2.1

	def distance(self, color):
		"""Returns the distance between the 2 colors seen from a r,b,g color cube"""
		rdistance = self.r - color.r
		gdistance = self.g - color.g
		bdistance = self.b - color.b
		return math.sqrt(rdistance*rdistance + gdistance*gdistance +bdistance*bdistance)


	def fromString(name):
		"""Return a Color object from a string"""
		assert(type(name) == types.StringType)
		if Color.__dict__.has_key(name):
			color = Color.__dict__[name]
		else:
			colors = [k for k in Color.__dict__.keys() if Color.__dict__[k].__class__ == Color]
			raise Exception, "unknown color: '" +name +"', choose between: " +str(colors)
		return color

	def getRGB(self):
		"""Return a (r,g,b) tuple"""
		return (self.r, self.g, self.b)


	def showAll():
		height,width = 12,2
		ColDev = height*width

		clrs = [k for k in Color.__dict__.keys() if Color.__dict__[k].__class__ == Color]
		dist = [Kplot.Color.fromString(k).distance(Kplot.Color.white) for k in clrs]
		clrs = [clrs[i] for i in Numeric.argsort(dist)]
		numCol = len(clrs)
		numDev = math.ceil(1.*numCol/ColDev)
		dark = [k for k in clrs if Kplot.Color.fromString(k).distance(Kplot.Color.white) > .6*math.sqrt(3)]
		colors = [[clrs[i+j*ColDev] for i in range(ColDev) if i+j*ColDev < numCol] for j in range(numDev)]
		
		for i in range(numDev):
			dev = Kplot.createDevice()
			panel = dev.createPanel()
			panel.setWorld((0,0),(width,height))
			b = 0
			h = height-1
			rects = []
			names = []
				
			for k in colors[i]:
				rect = Kplot.Rectangle((b,h),(b+1,h+1),color=k)
				if k in dark:
					lab  = Kplot.Text(k,(b+.5,h+.5), halign='center', color='white')
				else:
					lab  = Kplot.Text(k,(b+.5,h+.5), halign='center', color='black')
				rects.append(rect)
				names.append(lab)

				if b == width-1:
					b=0
					h-=1
				else:
					b+=1
			
			panel.add(rects,names)
			dev.output()
			dev.close()
			
				

	fromString = staticmethod(fromString)
	showAll = staticmethod(showAll)

#standard colors
Color.black = Color(0,0,0)
Color.blue = Color(0,0,1)
Color.green = Color(0,1,0)
Color.cyan = Color(0,1,1)
Color.red = Color(1,0,0)
Color.purple = Color(1,0,1)
Color.magenta = Color(1,0,1)
Color.yellow = Color(1,1,0)
Color.white = Color(1,1,1)

Color.orange = Color(1,0.6,0)
Color.lightcyan = Color(0.878,1.0,1.0)
Color.darkcyan = Color(0.0,0.545,0.545)
Color.maroon = Color(0.69,0.188,0.376)
Color.lightgreen = Color(0.565,0.933,0.565)
Color.navy = Color(0.0,0.0,0.502)
Color.darkslateblue = Color(0.282,0.239,0.545)
Color.slateblue = Color(0.416,0.353,0.804)
Color.mediumslateblue = Color(0.482,0.408,0.933)
Color.lightslateblue = Color(0.518,0.439,1.0)
Color.mediumblue = Color(0.0,0.0,0.804)
Color.skyblue = Color(0.529,0.808,0.922)
Color.lightblue = Color(0.678,0.847,0.902)
Color.turquoise = Color(0.251,0.878,0.816)
Color.darkgreen = Color(0.0,0.392,0.0)
Color.palegreen = Color(0.596,0.984,0.596)
Color.springgreen = Color(0.0,1.0,0.498)
Color.khaki = Color(0.941,0.902,0.549)
Color.lightyellow = Color(1.0,1.0,0.878)
Color.gold = Color(1.0,0.843,0.0)
Color.sienna = Color(0.627,0.322,0.176)
Color.salmon = Color(0.98,0.502,0.447)
Color.darkorange = Color(1.0,0.549,0.0)
Color.hotpink = Color(1.0,0.412,0.706)
Color.deeppink = Color(1.0,0.078,0.576)
Color.pink = Color(1.0,0.753,0.796)
Color.violet = Color(0.933,0.51,0.933)

#grey
Color.grey = Color(0.5, 0.5, 0.5)
Color.lightgrey = Color(0.75, 0.75, 0.75)
Color.darkgrey = Color(0.25, 0.25, 0.25)
Color.darkslategrey = Color(0.184,0.31,0.31)
Color.slategrey = Color(0.439,0.502,0.565)
Color.lightslategrey = Color(0.467,0.533,0.6)

Color.gray = Color.grey
Color.lightgray = Color.lightgrey
Color.darkgray = Color.darkgrey
Color.darkslategray = Color.darkslategrey
Color.slategray = Color.slategrey
Color.lightslategray = Color.lightslategrey

