import Kplot
import ppgplot
import types
from colortable import ColorTable
from Numeric import *
#from panel import Panel
from Kplot.tools import *
import os

class Device(Kplot.DeviceAbstract):
	def __init__(self, devicename=None, size=(150, 150), colors=32, units=None):
		Kplot.DeviceAbstract.__init__(self)
		if devicename == None:
			try:
				devicename = os.environ["KPLOT_PGPLOT_DEVICE"]
			except:
				raise Exception, "devicename not given, and KPLOT_PGPLOT_DEVICE env var not set"
		self.devicename = devicename
		self.size = size
		self.sizeunits = units
		self.colors = colors
		self.open()

		self.linestyles = {"noline"       :None,
				   "normal"       :1,
				   "dashed"       :2,
				   "dotted"       :4,
				   "dotdash"      :3,
				   "dashdotdotdot":5
				   }
		self.fillstyles = {"nofill"      :None,
				   "solid"       :1,
				   "hatched"     :3,
				   "crosshatched":4
				   }

		self.fonts = {"normal": 1,
			      "roman" : 2,
			      "italic": 3,
			      "script": 4
			      }

		#self.units = "points"

	def open(self):
		self.deviceID = ppgplot.pgopen(self.devicename)
		ppgplot.pgswin(0, 1, 0, 1)
		ppgplot.pgask(0)
		ppgplot.pgscr(1,0,0,0) #set black as foreground color (tnx cb!)
		ppgplot.pgscr(0,1,1,1) #set white as background color 
		self.setClipping(0)
		self.setDimensions()
		self.changeSize(self.size, self.sizeunits)
		self.basicColorTable = ColorTable(2, (self.colors-1)+2)
		ppgplot.pgscir((self.colors-1)+3, 255)
		self.imageColorMin, self.imageColorMax = ppgplot.pgqcir()
		pass

	def close(self, prompt=True):
		self.select()
		ppgplot.pgask(prompt)
		ppgplot.pgclos(self.deviceID)

	def setDimensions(self):
		dontcare1, width, dontcare2, height = ppgplot.pgqvsz(1) # inch
		self.dimensions["inch"] = (width, height)
		self.dimensions["points"] = (width*72, height*72)
		dontcare1, width, dontcare2, height = ppgplot.pgqvsz(2) # mm
		self.dimensions["mm"] = (width, height)
		self.dimensions["cm"] = (width/10.0, height/10.0)

	def changeSize(self, size, units=None):
		self.select()
		scales = {"inch":25.4,
			 "points":1.,
			  None:1.,
			 "mm":1.,
			 "cm":10.,
			  "normalised":150.}
		scale = scales[units]
		norm = self.getUnitMatrix(units) * size
		newdimension = self.getUnitMatrix("inch").inverse() * norm
		width, height = newdimension
		aspect = float(height) / width
		ppgplot.pgpap(width, aspect)
		ppgplot.pgpage()
		self.size = size
		self.sizeunits = units
		self.setDimensions()
		hardcopySize= (size[0]*scale,size[1]*scale)
		os.environ["KPLOT_HARDCOPY_SIZE"] = str(hardcopySize)
                     #write hardcopy size, so Device.hardcopy can pick it up 

	def setClipping(self, enable):
		self.select()
		ppgplot.pgsclp(enable)
		self.clipping = enable

	def getClipping(self):
		self.select()
		return ppgplot.pgqclp()

	def select(self):
		curDeviceID = ppgplot.pgqid()
		if(self.deviceID != curDeviceID):
			ppgplot.pgslct(self.deviceID)

	def clear(self):
		self.select()
		ppgplot.pgeras()

	def getDeviceMatrix(self):
		return Kplot.Matrix()

	def setViewport(self):
		self.select()
		p1 = self.getViewportMatrix() * (0, 0)
		p2 = self.getViewportMatrix() * (1, 1)
		(x1, y1), (x2, y2) = (p1, p2)
		ppgplot.pgsvp(x1, x2, y1, y2)

	def setPgLinestyle(self, linestyle):
		if linestyle not in self.linestyles:
			raise Kplot.CapsException, "Unknown linestyle: " +str(linestyle)
		else:
			pglinestyle = self.linestyles[linestyle]
			try:
				ppgplot.pgsls(pglinestyle)
			except:
				print "invalid linestyle '%s'" % linestyle
				raise

	def setPgFillstyle(self, fillstyle):
		if fillstyle not in self.fillstyles:
			raise Kplot.CapsException, "Unknown fillstyle: " +str(fillstyle)
		else:
			pgfillstyle = self.fillstyles[fillstyle]
			ppgplot.pgsfs(pgfillstyle)

	def setPgLinewidth(self, linewidth, units):
		"""when the units of linewidth are None, and the linewidth is default
		(i.e. 0.5), the pgplot linewidth is overruled and set to 1, because there
		is a discrepancy between the screen device and the hardcopy device.
		"""
		v = Kplot.Vector(linewidth, linewidth)
		nv = self.getUnitMatrix(units) * v
		v = self.getUnitMatrix("inch").inverse() * nv
		v = v.scale(200)
		linewidth = min(v.x, v.y)
       		linewidth = max(int(linewidth+0.5), 1)
		if self.devicename != '/xwin' and nv.x <= 0.005 and units == None:
			linewidth = 1 #overruling linewidth
		ppgplot.pgslw(linewidth)

	def getPgColor(self, color):
		if type(color) == types.StringType:
			if color.lower() in ["bg", "background"]:
				colorindex = 0
			elif color.lower() in ["fg", "foreground"]:
				colorindex = 1
			else:
				color = Kplot.Color.fromString(color)
				colorindex = self.basicColorTable.findIndex(color)
		else:
				colorindex = self.basicColorTable.findIndex(color)

		if colorindex == None:
			if self.basicColorTable.isFull():
				print "not enough basic colors, using nearest color"
				colorindex = self.basicColorTable.findIndex(self.basicColorTable.getClosest(color))
			else:
				colorindex = self.basicColorTable.addColor(color, ppgplot)

		return colorindex

	def setPgColor(self, color):
		colorindex = self.getPgColor(color)
		try:
			ppgplot.pgsci(colorindex)
		except:
			print "invalid color '%s' index: %i " % (color, colorindex)
			raise

	def setPgBgColor(self, color):
		colorindex = self.getPgColor(color)
		try:
			ppgplot.pgstbg(colorindex)
		except:
			print "invalid color '%s' index: %i " % (color, colorindex)
			raise

	def setPgFont(self, font):
		if font not in self.fonts:
			raise Kplot.CapsException, "Unknown font: " +str(font)
		else:
			pgfont = self.fonts[font]
			try:
				ppgplot.pgscf(pgfont)
			except:
				print "invalid font '%s'" % font
				raise

	def setPgFontsize(self, fontsize, units):
		v = Kplot.Vector(0, fontsize)
		nv = self.getUnitMatrix(units) * v
		height = nv.y * 40
		#print fontsize, height
		ppgplot.pgsch(height)


	def plotPolyline(self, x, y, linestyle, linewidth, color, units=None):
		self.setViewport()
		self.setClipping(self.clipping)
		#x, y = self.transformation.transform(x, y)
		x, y = self.getWorldMatrix().mulXY(x, y)
		#print x, y, self.getClipping()
		if linestyle != "noline":
			self.setPgLinestyle(linestyle)
			self.setPgColor(color)
			self.setPgLinewidth(linewidth, units)
			ppgplot.pgline(x, y)

	def plotPolygon(self, x, y, fillstyle, linestyle, linewidth, color, units=None):
		self.setViewport()
		self.setClipping(self.clipping)
		#x, y = self.transformation.transform(x, y)
		x, y = self.getWorldMatrix().mulXY(x, y)
		if fillstyle != "nofill":
			self.setPgFillstyle(fillstyle)
			self.setPgLinestyle("normal")
			self.setPgColor(color)
			ppgplot.pgslw(1)
			#self.setPgLinewidth(linewidth, "mm")
			ppgplot.pgpoly(x, y)


		if linestyle != "noline":
			px = concatenate((x, x[0:1]))
			py = concatenate((y, y[0:1]))
			self.setPgLinestyle(linestyle)
			self.setPgColor(color)
			self.setPgLinewidth(linewidth, units)
			ppgplot.pgline(px, py)


	#def plotArc(self, p1, p2, fillstyle, linestyle, linewidth, color, startangle, anglelength, units=None):
	#	self.select()
	#	self.setViewport()
	#	self.setClipping(self.clipping)


	def plotText(self, text, location, font, weight, fontsize, color, bgcolor, textangle, valign, halign, units=None):
		self.setViewport()
		self.setClipping(self.clipping)
		textangle = todegrees(textangle)
		#ppgplot.pgswin(0, 1, 0, 1)
		#location = self.transformation.transformPoint(location)
		location = self.getWorldMatrix() * location

		self.select()
		self.setPgLinestyle("normal")
		self.setPgColor(color)
		self.setPgLinewidth(weight, units)
		self.setPgFontsize(fontsize, units)
		self.setPgFont(font)
		#ppgplot.pgscf(1)
		#ppgplot.pgsch(fontsize/5)

		if bgcolor == None:
			ppgplot.pgstbg(-1)
		else:
			self.setPgBgColor(bgcolor)

		p1, p2, p3, p4 = ppgplot.pgqtxt(0, 0, 0, 0.0, "a")
		p1 = Kplot.Vector(p1)
		p2 = Kplot.Vector(p2)
		p3 = Kplot.Vector(p3)
		p4 = Kplot.Vector(p4)
		#print p1
		#corrleft = -p1.x
		#corrup = -p1.y + (p1

		base = Kplot.Vector(p1)
		#base.y = 0
		#print base
		#base = self.transformation.transformPoint(base)
		#base = self.getViewportMatrix().inverse() * base

		#print base, self.getUnitMatrix("mm").inverse() * base
		#print "corr", corr
		#corr = Kplot.Vector(0, 0) - Kplot.Vector(-p2.x, p2.y)


		x, y = location
		p1, p2, p3, p4 = ppgplot.pgqtxt(x, y, textangle, 0.0, text)
		p1 = Kplot.Vector(p1)
		p2 = Kplot.Vector(p2)
		p3 = Kplot.Vector(p3)
		p4 = Kplot.Vector(p4)
		dirup = (p2 - p1)
		dirright = (p4 - p1)

		#print dirup, dirright
		#halign = "left"
		#valign = "bottom"

		#print halign, valign
		temp = location
		if halign == "left":
			pass
		elif halign == "right":
			location -= dirright
		elif halign == "center":
			location -= dirright.scale(0.5)
		else:
			raise Exception, "invalid halign '%s' " % (halign)

		if valign == "bottom":
			pass
		elif valign == "top":
			location -= dirup
		elif valign == "center":
			location -= dirup.scale(0.5)
		else:
			raise Exception, "invalid valign '%s' " % (valign)

		#location = temp

		location = Kplot.Vector(location) - base #Kplot.Vector(base.x, base.y)
		x, y = location
		#x += corrleft
		#y += corrup
		ppgplot.pgptxt(x, y, textangle, 0, text)



	def plotImage(self, data, colormap, datamin, datamax, function ,origin=(1,1)):
		from Kplot.matrix import Matrix
		self.setViewport()
		worldmatrix = self.getWorldMatrix()
		delx,dely = origin
		worldmatrix = worldmatrix.inverse()
		wm = worldmatrix.matrix
		wm[0][2] += delx
		wm[1][2] += dely
		worldmatrix.matrix = wm
		worldmatrix = worldmatrix.inverse()
		wm = worldmatrix.matrix

		matrix = array([wm[0][2], wm[0][0], wm[0][1], wm[1][2], wm[1][0], wm[1][1]])
		if datamin > datamax:
			datamin, datamax = datamax, datamin

		colorwidth = (self.imageColorMax - self.imageColorMin) + 1
		data = clip(data, datamin, datamax)
		scale = colorwidth / float((datamax - datamin))
		data = ((data - datamin) * scale)
		for i in range(colorwidth):
			index = float(i)/(colorwidth-1)
			index = function(index)
			index = clip(index, 0.0, 1.0)
			color = colormap(index)
			ppgplot.pgscr(i + self.imageColorMin, color.r, color.g, color.b)

		rows = data.shape[0]
		columns = data.shape[1]
		columns, rows = rows, columns
		(x1, y1), (x2, y2) = (0, 0), (rows-1, columns-1)

		#print "-", columns, rows, x1, x2, y1, y2

		ppgplot.pgimag(data, columns, rows, x1, x2, y1, y2, colorwidth-1, 0, matrix)

	def plotContour(self, data, level, color, linestyle, linewidth, origin, labeling, labelstart, labelinterval,
			labelcolor, font, fontsize, labellinewidth, blank, units=None):
		level = array([level])
		from Kplot.matrix import Matrix
		self.setViewport()
		worldmatrix = self.getWorldMatrix()
		delx,dely = origin
		worldmatrix = worldmatrix.inverse()
		wm = worldmatrix.matrix
		wm[0][2] += delx
		wm[1][2] += dely
		worldmatrix.matrix = wm
		worldmatrix = worldmatrix.inverse()
		wm = worldmatrix.matrix

		matrix = array([wm[0][2], wm[0][0], wm[0][1], wm[1][2], wm[1][0], wm[1][1]])

		rows = data.shape[0]
		columns = data.shape[1]
		columns, rows = rows, columns
		(x1, y1), (x2, y2) = (0, 0), (rows-1, columns-1)

		self.setPgLinestyle(linestyle)
		self.setPgColor(color)
		self.setPgLinewidth(linewidth, units)


		ppgplot.pgconb(data, columns, rows,  x1, x2, y1, y2, level, 1   , matrix, blank)
		if labeling:
			self.setPgLinestyle("normal")
			self.setPgColor(labelcolor)
			self.setPgLinewidth(labellinewidth, units)
			self.setPgFontsize(fontsize, units)
			self.setPgFont(font)
			labelstring = '%g' % level[0]
			ppgplot.pgconl(data, columns, rows,  x1, x2, y1, y2, level[0], matrix, labelstring, labelinterval, labelstart)


	def getLinestyles(self):
		return self.linestyles.keys()

	def getFillstyles(self):
		return self.fillstyles.keys()

	def getFonts(self):
		return self.fonts.keys()

	def getPos(self, clip=1):
		self.select()
		ppgplot.pgsvp(0, 1, 0, 1)
		validrange = 0
		while not validrange:
			x, y, char = ppgplot.pgband(0, 0, 0, 0)
			#print ord(char)
			if ord(char) == 27:
				return None
			if x >= 0.0 and x <= 1.0 and y >= 0.0 and y <= 1.0 or not clip:
				validrange = 1
		return (x, y, char)

	def setBgColor(self, color):
		if not hasattr(color, "r"):
			color = Kplot.Color.fromString(color)
		ppgplot.pgscr(0,color.r, color.g, color.b)

	def setFgColor(self, color):
		if not hasattr(color, "r"):
			color = Kplot.Color.fromString(color)
		ppgplot.pgscr(1,color.r, color.g, color.b)


	def plotLines(self, x, y, linestyle, linewidth, color, units=None):
		self.setViewport()
		self.setClipping(self.clipping)
		#x, y = self.transformation.transform(x, y)
		x, y = self.getWorldMatrix().mulXY(x, y)
		#print x, y, self.getClipping()
		if linestyle != "noline":
			self.setPgLinestyle(linestyle)
			self.setPgColor(color)
			self.setPgLinewidth(linewidth, units)
			#ppgplot.pgline(x, y)
			for i in range(len(x) / 2):
				ppgplot.pgmove(x[i*2], y[i*2])
				ppgplot.pgdraw(x[i*2+1], y[i*2+1])

	def plotCircle(self, center, radius, fillstyle, linestyle, linewidth, color, units=None):
		self.setViewport()
		self.setClipping(self.clipping)
		#wm = self.getWorldMatrix().matrix
		#print wm
		#matrix = array([wm[0][2], wm[0][0], wm[0][1], wm[1][2], wm[1][0], wm[1][1]])
		wm = self.getWorldMatrix().inverse()
		p1 = wm * (0,0)
		p2 = wm * (1,1)
		ppgplot.pgswin(p1.x, p2.x, p1.y, p2.y)
		#print p1, p2, radius

		#x, y = self.transformation.transform(x, y)
		#x, y = self.getWorldMatrix() * center
		x, y = center
		#radius = self.getWorldMatrix() * center
		if fillstyle != "nofill":
			self.setPgFillstyle(fillstyle)
			self.setPgLinestyle("normal")
			self.setPgColor(color)
			ppgplot.pgslw(1)
			#self.setPgLinewidth(linewidth, "mm")
			ppgplot.pgcirc(x, y, radius)


		if linestyle != "noline":
			ppgplot.pgsfs(2) # outline
			self.setPgLinestyle(linestyle)
			self.setPgColor(color)
			self.setPgLinewidth(linewidth, units)
			ppgplot.pgcirc(x, y, radius)
		ppgplot.pgswin(0,1,0,1)

class WindowDevice(Device):
	def __init__(self, size=(150, 150), colors=32, units=None):
		Device.__init__(self, devicename="/xwin", size=size,
			colors=colors, units=units)

class GifPortraitDevice(Device):
	def __init__(self, filename=None, size=(150, 150), colors=32, units=None):
		if filename == None:
			filename = os.environ.get("KPLOT_FILENAME", "Kplot")
		Device.__init__(self, devicename=filename+".gif/GIF", size=size,
			colors=colors, units=units)

class GifLandscapeDevice(Device):
	def __init__(self, filename=None, size=(150, 150), colors=32, units=None):
		if filename == None:
			filename = os.environ.get("KPLOT_FILENAME", "Kplot")
		Device.__init__(self, devicename=filename+".gif/VGIF", size=size,
			colors=colors, units=units)

class PsLandscapeDevice(Device):
	def __init__(self, filename=None, size=(150, 150), colors=32, units=None):
		if filename == None:
			filename = os.environ.get("KPLOT_FILENAME", "Kplot")
		Device.__init__(self, devicename=filename+".eps/CPS", size=size,
			colors=colors, units=units)

class PsPortraitDevice(Device):
	def __init__(self, filename=None, size=(150, 150), colors=32, units=None):
		if filename == None:
			filename = os.environ.get("KPLOT_FILENAME", "Kplot")
		Device.__init__(self, devicename=filename+".eps/VCPS", size=size,
			colors=colors, units=units)

class PsLandscapeDeviceBw(Device):
	def __init__(self, filename=None, size=(150, 150), colors=32, units=None):
		if filename == None:
			filename = os.environ.get("KPLOT_FILENAME", "Kplot")
		Device.__init__(self, devicename=filename+".eps/PS", size=size,
			colors=colors, units=units)

class PsPortraitDeviceBw(Device):
	def __init__(self, filename=None, size=(150, 150), colors=32, units=None):
		if filename == None:
			filename = os.environ.get("KPLOT_FILENAME", "Kplot")
		Device.__init__(self, devicename=filename+".eps/VPS", size=size,
			colors=colors, units=units)

class NullDevice(Device):
	def __init__(self, size=(150, 150), colors=32, units=None):
		Device.__init__(self, devicename="/NULL", size=size, colors=colors, units=units)
