import types
import Kplot
import Numeric 
from panel import Panel

class Device(Kplot.DeviceAbstract):
	def __init__(self, size=(72*10, 72*10), filename="Kplot.ps"):
		Kplot.DeviceAbstract.__init__(self)
		self.size = size
		self.filename = filename
		self.linestyles = ["normal", "noline"]
		self.fillstyles = ["solid", "nofill"]


	def getPanel(self, p1=(0.1,0.1), p2=(0.9, 0.9), parent=None):
		if parent == None:
			parent = self
		panel = Panel(p1=p1, p2=p2, parent=parent)
		if parent == self:
			self.children.append(panel)
		return panel

	def getDeviceMatrix(self):
		width, height = self.size
		return Kplot.Matrix.scaleXY(width, height)

	def output(self):
		self.file = file(self.filename, "wb")
		self.outputHeader()

		self.pushClipPath()
		Kplot.DeviceAbstract.output(self)
		self.popClipPath()
		self.outputFooter()
		self.file.close()
		

	def outputHeader(self):
		self.file.write("%!\n")

	def outputFooter(self):
		self.file.write("showpage\n")

	def outputMatrix(self):
		code = ""
		m = self.getTotalMatrix().matrix
		#code += "[%f %f %f %f %f %f] setmatrix\n" % (m[0][0], m[0][1], m[1][0], m[1][1], m[0][2], m[1][2])
		code += "initmatrix\n[%f %f %f %f %f %f] concat\n" % (m[0][0], m[1][0], m[0][1], m[1][1], m[0][2], m[1][2])
		#code += "[%f %f %f %f %f %f] setmatrix\n" % (m[0][0], m[1][0], m[0][1], m[1][1], m[1][2], m[0][2])
		self.write(code)

	def outputColor(self, color):
		if type(color) == types.StringType:
			ncolor = Kplot.Color.fromString(color)
		else:
			ncolor = color
		r, g, b = ncolor.getRGB()
		code = "%f %f %f setrgbcolor\n" % (r, g, b)
		self.file.write(code)

	def outputFillstyle(self, fillstyle):
		if fillstyle == "solid":
			self.writeln("gsave\nfill\ngrestore")
		elif fillstyle == "nofill":
			pass
		else:
			raise Kplot.CapsException, "Unknown fillstyle: " +str(fillstyle)


	def outputLinestyle(self, linestyle):
		if linestyle == "normal":
			self.writeln("stroke")
		elif linestyle == "noline":
			self.writeln("newpath")
		else:
			raise Kplot.CapsException, "Unknown linestyle: " +str(linestyle)

	def write(self, code):
		self.file.write(code)

	def writeln(self, code):
		self.file.write(code+"\n")

	def plotPolyline(self, x, y, linestyle, linewidth, color):
		print color
		x, y = self.transformation.transform(x, y)
		self.outputMatrix()
		self.writeln("%f setlinewidth \n" % linewidth)
		code = ""
		length = min(len(x), len(y))
		code += "%f %f moveto\n" % (x[0], y[0])
		for i in xrange(1, length):
			code += "%f %f lineto\n" % (x[i], y[i])

		self.outputColor(color)
		self.write(code)
		self.writeln("initmatrix")

		self.outputLinestyle(linestyle)


	def plotPolygon(self, x, y, fillstyle, linestyle, linewidth, color):
		x, y = self.transformation.transform(x, y)
		self.outputMatrix()
		length = min(len(x), len(y))
		self.writeln("%f %f moveto" % (x[0], y[0]))
		for i in xrange(1, length):
			self.writeln("%f %f lineto" % (x[i], y[i]))

		self.writeln("%f setlinewidth" % linewidth)
		self.writeln("closepath")
		self.writeln("initmatrix")
		self.outputColor(color)
		self.outputFillstyle(fillstyle)
		self.outputLinestyle(linestyle)

	def plotArc(self, p1, p2, fillstyle, linestyle, linewidth, color, startangle, anglelength):
		x, y = self.transformation.transform(x, y)
		self.outputMatrix()
		#self.writeln("arc")
		#self.outputColor(color)
		#self.writeln("gsave")
		#self.outputFillstyle(fillstyle)
		#self.writeln("grestore")
		#self.outputLinestyle(linestyle)
		x1, y1 = p1
		x2, y2 = p2
		width = x2 - x1
		height = y2 - y1

		#code = ""
		#m = self.getTotalMatrix().matrix
		#code += "[%f %f %f %f %f %f] setmatrix\n" % (m[0][0], m[0][1], m[1][0], m[1][1], m[0][1], m[0][2])
		#self.file.write(code)

		x = x1 + width/2.0
		y = y1 + height/2.0
		radius = max(width, height) / 2
		m = Kplot.Matrix.scaleXY(width/radius, height/radius).matrix
		#self.writeln("[%f %f %f %f %f %f] concat" % (m[0][0], m[0][1], m[1][0], m[1][1], m[0][1], m[0][2]))
		print x, y, radius

		self.writeln("%f %f moveto" % (x, y))
		self.writeln("%f %f %f %f %f arc" % (x, y, radius, startangle, anglelength))
		self.writeln("closepath")
		self.writeln("initmatrix")
		self.outputColor(color)
		self.outputFillstyle(fillstyle)
		#self.writeln("fill")

		#self.writeln("gsave")
		#self.
		#self.writeln("closepath")
		#self.outputFillstyle(fillstyle)
		#self.writeln("grestore")
		#self.outputLinestyle(linestyle)

		pass

	def plotText(self, text, location, font, fontsize, color, bgcolor, textangle, valign, halign):
		#self.outputMatrix()
		code = ""
		self.writeln(code)

	def plotImage(self, data, colormap, datamin, datamax, function ,origin=(1,1)):
		self.outputMatrix()
		if datamin > datamax:
			datamin, datamax = datamax, datamin
		data = Numeric.clip(data, datamin, datamax)
		scale = 255.0 / (datamax - datamin)
		data = ((data - datamin) * scale)
		lutstr = "<"
		for i in range(0, 256):
			index = i/255.0
			index = function(index)
			color = colormap.getColor(index)
			r = color.r * 255
			g = color.g * 255
			b = color.b * 255
			lutstr += "%0.2x%0.2x%0.2x " % (r, g, b)
		lutstr = lutstr[:-1]
		lutstr += ">"
		height, width = data.shape
		self.writeln("[/Indexed /DeviceRGB 255 %s] setcolorspace" % lutstr)
		self.writeln("<< /ImageType 1 /Width %i /Height %i /BitsPerComponent 8 /Decode [0 255] /ImageMatrix "\
		"[1 0 0 1 0 0] /DataSource currentfile /ASCIIHexDecode filter >> image" % (width, height))

		datastr = ""
		print width, height
		for y in range(height):
			for x in range(width):
				#datastr += "%0.2x" % data[(height-1)-y][x]
				datastr += "%0.2x" % data[y][x]
			self.writeln(datastr)
			datastr = ""

		self.writeln(datastr)


	def pushClipPath(self):
		#return
		self.writeln("gsave")
		viewportmatrix = self.getViewportMatrix()
		devicematrix = self.getDeviceMatrix()
		matrix = devicematrix * viewportmatrix
		print "clipping with: ", viewportmatrix.matrix
		p0 = matrix * (0, 0)
		p1 = matrix * (0, 1)
		p2 = matrix * (1, 1)
		p3 = matrix * (1, 0)
		m = viewportmatrix.matrix
		matrixstr = "%f %f %f %f %f %f" % (m[0][0], m[1][0], m[0][1], m[1][1], m[0][2], m[1][2])
		self.writeln("%% clipping with viewport matrix: %s " % (str(matrixstr)))
		self.writeln("%f %f moveto" % (p0[0], p0[1]))
		self.writeln("%f %f lineto" % (p1[0], p1[1]))
		self.writeln("%f %f lineto" % (p2[0], p2[1]))
		self.writeln("%f %f lineto" % (p3[0], p3[1]))
		self.writeln("closepath")
		self.writeln("clip")
		self.writeln("newpath")

	def pushReverseClipPath(self):
		#return
		self.writeln("gsave")
		viewportmatrix = self.getViewportMatrix()
		devicematrix = self.getDeviceMatrix()
		matrix = devicematrix * viewportmatrix
		p0 = matrix * (0, 0)
		p1 = matrix * (0, 1)
		p2 = matrix * (1, 1)
		p3 = matrix * (1, 0)
		m = viewportmatrix.matrix
		matrixstr = "%f %f %f %f %f %f" % (m[0][0], m[1][0], m[0][1], m[1][1], m[0][2], m[1][2])
		self.writeln("%% clipping reverse with viewport matrix: %s " % (str(matrixstr)))
		self.writeln("clippath")
		self.writeln("reversepath")
		self.writeln("%f %f moveto" % (p0[0], p0[1]))
		self.writeln("%f %f lineto" % (p1[0], p1[1]))
		self.writeln("%f %f lineto" % (p2[0], p2[1]))
		self.writeln("%f %f lineto" % (p3[0], p3[1]))
		self.writeln("closepath")
		self.writeln("clip")
		self.writeln("newpath")


	def popClipPath(self):
		#return
		self.writeln("grestore")

	#def getDeviceMatrix(self):

