import types
import Kplot
import Numeric
import os
from Kplot.tools import abstract
from string import join
from Kplot import Matrix


_fillStyleCode = """<<
	/PatternType 1
	/PaintType 2
	/TilingType 1
	/BBox [0 0 10 10]
	/XStep 10
	/YStep 10
	/PaintProc
	{	begin
			10 setlinewidth
			0 0 moveto
			10 10 lineto
			stroke
			10 0 moveto
			0 10 lineto
			stroke
		end
	}
>>

matrix
makepattern

/CrossHatch exch def

<<
	/PatternType 1
	/PaintType 2
	/TilingType 1
	/BBox [0 0 10 10]
	/XStep 10
	/YStep 10
	/PaintProc
	{	begin
			10 setlinewidth
			0 0 moveto
			10 10 lineto
			stroke
		end
	}
>>
matrix
makepattern

/Hatch exch def
"""

def topoints(mm):
	return ((mm / 10) / 2.54) * 72

class PsBaseDevice(Kplot.DeviceAbstract):
	def __init__(self, size=(200, 200), units=None):
		Kplot.DeviceAbstract.__init__(self)
		self.linestyles = ["noline", "normal", "dashed", "dotted"]
		self.fillstyles = ["solid", "nofill", "hatched", "crosshatched"]
		self.fontMap = {"normal": "Helvetica", "roman" : "Times-Roman", "italic" : "Times-Italic"}
		self.fgColor = Kplot.Color(0, 0, 0)
		self.bgColor = None
		self.size = Kplot.Vector(size)
		self.sizeunits = units
		self.dpi = 72
		self.open();

	def open(self):
		self.res = (1.0, 1.0)
		self.setDimensions()
		norm = self.getUnitMatrix(self.sizeunits) * self.size
		self.res = self.getUnitMatrix("inch").inverse() * norm
		self.res = self.res.scale(self.dpi);
		#self.outpipe, self.inpipe = os.popen2("gs -dNOPAUSE -r%i -g%ix%i -" % (self.dpi, self.res.x, self.res.y))
		#self.setDimensions()
		#self.outpipe = file("blaat.ps", "w")


	def setDimensions(self):
		width, height = self.res
		self.dimensions["inch"] = (width/72, height/72)
		self.dimensions["points"] = (width, height)
		mmwidth, mmheight = (width/72.0) * 25.4, (height/72.0) * 25.4
		self.dimensions["mm"] = (mmwidth, mmheight)
		self.dimensions["cm"] = (mmwidth/10.0, mmheight/10.0)

	def createPanel(self, p1=(0.1,0.1), p2=(0.9, 0.9), parent=None):
		if parent == None:
			parent = self
		panel = Kplot.Panel(p1=p1, p2=p2, parent=parent)
		if parent == self:
			self.children.append(panel)
		return panel

	def getDeviceMatrix(self):
		width, height = self.res
		return Kplot.Matrix.scaleXY(width, height)

	def preOutput(self):
		self.pushClipPath()
		if self.bgColor != None:
			self.plotPolygon(x=[0,1,1,0], y=[0,0,1,1], linestyle="noline",
			fillstyle="solid", linewidth=1, color=self.bgColor, units="mm")

	def postOutput(self):
		self.popClipPath()
		self.outputFooter()

	def write(self, code):
		abstract()

	def writeln(self, code):
		abstract()

	def flush(self):
		abstract();

	def clear(self):
		self.outputHeader()
		self.outputHeaderCode();

	def outputHeader(self):
		self.write("%!\n")

	def outputHeaderCode(self):
		self.write("""/hleftalign { 1 index  6 index sub } bind def
/hrightalign { 1 index  4 index sub } bind def
/hmiddlealign { 1 index 4 index 7 index sub 2 div sub } bind def
/hbasealign { 1 index } bind def

/vbottomalign { 0 index  5 index sub } bind def
/vtopalign { 0 index  3 index sub } bind def
/vmiddlealign { 0 index 3 index 6 index sub 2 div sub } bind def
/vbasealign { 0 index } bind def
""");
		global _fillStyleCode
		self.write(_fillStyleCode)

	def outputFooter(self):
		#self.write("showpage\n")
		pass

	def outputMatrix(self):
		code = ""
		worldmatrix = self.getWorldMatrix()
		viewportmatrix = self.getViewportMatrix()
		devicematrix = self.getDeviceMatrix()
		matrix = devicematrix * (viewportmatrix * worldmatrix)
		m = matrix.matrix
		#print devicematrix
		#print viewportmatrix
		#print worldmatrix
		#print matrix
		#print "--- next ---"
		#code += "[%f %f %f %f %f %f] setmatrix\n" % (m[0][0], m[0][1], m[1][0], m[1][1], m[0][2], m[1][2])
		code += "initmatrix\n[%f %f %f %f %f %f] concat\n" % (m[0][0], m[1][0], m[0][1], m[1][1], m[0][2], m[1][2])
		#code += "[%f %f %f %f %f %f] setmatrix\n" % (m[0][0], m[1][0], m[0][1], m[1][1], m[1][2], m[0][2])
		self.write(code)

	def outputColor(self, color):
		if color in ["background", "bg"]:
			if self.bgColor == None:
				ncolor = Kplot.Color(1,1,1)
			else:
				ncolor = self.bgColor
		elif color in ["foreground", "fg"]:
			ncolor =self.fgColor
		elif type(color) == types.StringType:
			ncolor = Kplot.Color.fromString(color)
		else:
			ncolor = color
		r, g, b = ncolor.getRGB()
		code = "%f %f %f setrgbcolor\n" % (r, g, b)
		self.write(code)

	def outputFillstyle(self, fillstyle):
		if fillstyle == "solid":
			self.writeln("gsave\nfill\ngrestore")
		elif fillstyle == "outlined":
			self.writeln("gsave\nstroke\ngrestore")
		elif fillstyle == "hatched":
			self.writeln("gsave currentrgbcolor Hatch setpattern fill grestore");
		elif fillstyle == "crosshatched":
			self.writeln("gsave currentrgbcolor CrossHatch setpattern fill grestore");
		elif fillstyle == "nofill":
			#self.writeln("gsave newpath grestore");
			pass
		else:
			raise Kplot.CapsException, "Unknown fillstyle: '%s'" % str(fillstyle)


	def outputLinestyle(self, linestyle):
		if linestyle == "normal":
			self.writeln("[] 0 setdash")
			self.writeln("stroke")
		elif linestyle == "dashed":
			self.writeln("[10 5] 0 setdash")
			self.writeln("stroke")
		elif linestyle == "dotted":
			self.writeln("[1] 0 setdash")
			self.writeln("stroke")
		elif linestyle == "noline":
			self.writeln("newpath")
		else:
			raise Kplot.CapsException, "Unknown linestyle: " +str(linestyle)

	def outputLinewidth(self, linewidth, units):
		v = Kplot.Vector(linewidth, linewidth)
		nv = self.getUnitMatrix(units) * v
		v = self.getUnitMatrix("points").inverse() * nv
		linewidth = min(v.x, v.y)
		self.writeln("%f setlinewidth" % linewidth)


	def plotPolyline(self, x, y, linestyle, linewidth, color, units=None):
		#print color
		#x, y = self.transformation.transform(x, y)
		#x, y = self.getTotalMatrix().mulXY(x, y)
		self.setClippath()
		self.outputMatrix()
		#self.writeln("initmatrix")
		#self.writeln("%f setlinewidth \n" % linewidth)
		code = ""
		length = min(len(x), len(y))
		code += "%f %f moveto\n" % (x[0], y[0])
		for i in xrange(1, length):
			code += "%f %f lineto\n" % (x[i], y[i])

		self.outputColor(color)
		self.write(code)
		self.writeln("initmatrix")

		self.outputLinewidth(linewidth, units)
		self.outputLinestyle(linestyle)
		self.writeln("clear")
		self.flush();
		#self.write("showpage\n")


	def plotPolygon(self, x, y, fillstyle, linestyle, linewidth, color, units=None):
		self.setClippath()
		#x, y = self.transformation.transform(x, y)
		#print x, y
		self.outputMatrix()
		length = min(len(x), len(y))
		self.writeln("%f %f moveto" % (x[0], y[0]))
		for i in xrange(1, length):
			self.writeln("%f %f lineto" % (x[i], y[i]))

		self.writeln("closepath")
		self.writeln("initmatrix")
		self.outputColor(color)
		self.outputFillstyle(fillstyle)
		self.outputLinewidth(linewidth, units)
		self.outputLinestyle(linestyle)
		self.flush();

	def plotText(self, text, location, font, fontsize, weight, color, bgcolor, textangle, valign, halign, units=None):
		self.setClippath()
		#self.setClippath()
		self.psCodeValign = {"top":"vtopalign", "bottom":"vbottomalign", "center":"vmiddlealign", "base":"vbasealign"}
		self.psCodeHalign = {"left":"hleftalign", "right":"hrightalign", "center":"hmiddlealign", "base":"hbasealign"}
		valign, halign = self.psCodeValign[valign], self.psCodeHalign[halign]
		self.outputColor(color)
		v = Kplot.Vector(fontsize, fontsize)
		v = self.getUnitMatrix(units) * v
		v = self.getUnitMatrix("points").inverse() * v

		location = self.getTotalMatrix() * location

		textangledegrees = Kplot.todegrees(textangle)
		self.outputMatrix()
		if font not in self.fontMap:
			s = "font '%s' not supported, choose between: %s" % (font, join(self.fontMap.keys(), ", "))
			raise Exception, s
		fontname = self.fontMap[font]
		self.writeln("""initmatrix
			/%s findfont
			 %s scalefont setfont""" % (fontname, v.x) );
		self.writeln("""0 0 moveto
			(%s)
			false charpath
			pathbbox
			%f %f
			newpath
			%s
			7 1 roll
			%s
			8 7 roll
			%f %f translate
			%f rotate
			%f %f translate
			moveto
			(%s)
			show""" %(text, location[0], location[1], valign, halign,
							location[0], location[1], textangledegrees,
							-location[0], -location[1], text))
		self.writeln("clear")
		self.flush()

	def plotImage(self, data, colormap, datamin, datamax, function, origin=(1,1)):
		self.setClippath()
		self.outputMatrix()
		if datamin > datamax:
			datamin, datamax = datamax, datamin
		data = Numeric.clip(data, datamin, datamax)
		scale = 255.0 / (datamax - datamin)
		data = ((data - datamin) * scale)
		lutstr = "<"
		for i in range(0, 256):
			index = i/256.0
			index = function(index)
			index = Numeric.clip(index, 0.0, 1.0)
			color = colormap(index)
			r = color.r * 255
			g = color.g * 255
			b = color.b * 255
			lutstr += "%0.2x%0.2x%0.2x " % (r, g, b)
		lutstr = lutstr[:-1]
		lutstr += ">"
		height, width = data.shape
		line = str(origin[0]-.5)+" "+str(origin[1]-.5)+" translate" #shift image to fit panel
		self.writeln(line)
		self.writeln("[/Indexed /DeviceRGB 255 %s] setcolorspace" % lutstr)
		self.write("<< /ImageType 1 /Width %i /Height %i /BitsPerComponent 8 /Decode [0 255] /ImageMatrix "\
		"[1 0 0 1 0 0] /DataSource currentfile /ASCIIHexDecode filter /FlateDecode filter >> image\n" % (width, height))

		datastr = ""
		hexstring = data.flat.astype(Numeric.Int8).tostring();
		s = hexstring.encode("zlib").encode("hex")
		self.writeln(s)
		#self.writeln(" clear")
		self.flush()

	def __plotLines(self, x, y, linestyle, linewidth, color, units=None):
		self.setClippath()
		self.outputMatrix()
		#self.writeln("%f setlinewidth \n" % linewidth)
		if linestyle != "noline":
			code = ""
			length = min(len(x), len(y)) /2
			#for i in xrange(length):
			#	code += "%f %f moveto\n" % (x[i*2], y[i*2])
			#	code += "%f %f lineto\n" % (x[i*2+1], y[i*2+1])
			num = Numeric
			flat = num.array([x,y])
			flat = num.ravel(num.transpose(num.array([x,y])))
			#flat = num.ravel(num.transpose(num.array([x,y], Float32)))
			arstr = flat.astype(num.Float32).byteswapped().tostring().encode("hex")
			codestr = num.ravel(num.transpose(
							num.array([num.ones(length, num.Int8), num.ones(length, num.Int8)+2]))
				).astype(num.Int8).tostring().encode("hex")
			#print "blaat", length, len(arstr)
			self.write("{ <95 30") # homogenous array
			self.write("%04x " % (length*4))
			self.write(arstr)
			self.write("> <")
			self.write(codestr)
			self.write(">} ustroke")
			self.write("\n")
			#self.writeln("%i {moveto lineto} repeat" % length)
			#self.write(code)
			#self.writeln("initmatrix")
			#self.outputColor(color)
			#self.outputLinewidth(linewidth, units)
			#self.outputLinestyle(linestyle)
			self.flush()
	
	def plotLines(self, x, y, linestyle, linewidth, color, units=None):
		self.setClippath()
		self.outputMatrix()
		length = min(len(x), len(y)) /2
		if (linestyle != "noline") and (length > 0):
			for i in xrange(length):
				self.write("%f %f moveto\n%f %f lineto\n" % (x[i*2], y[i*2],x[i*2+1], y[i*2+1]))
			self.writeln("initmatrix")
			self.outputColor(color)
			self.outputLinewidth(linewidth, units)
			self.outputLinestyle(linestyle)
			self.flush()

	def plotCircle(self, center, radius, fillstyle, linestyle, linewidth, color, units=None):
		self.setClippath()
		self.outputMatrix()
		x, y = center
		self.write("%f %f %f 0 360 arc\n" % (x, y, radius))
		self.writeln("initmatrix")
		self.outputColor(color)
		self.outputLinewidth(linewidth, units)
		self.outputFillstyle(fillstyle)
		self.outputLinestyle(linestyle)
		self.flush()

	def setClippath(self):
		self.writeln("initclip")
		if self.clipping:
			viewportmatrix = self.getViewportMatrix()
			devicematrix = self.getDeviceMatrix()
			matrix = devicematrix * viewportmatrix
		else:
			matrix = self.getDeviceMatrix()
		p0 = matrix * (0, 0)
		p1 = matrix * (0, 1)
		p2 = matrix * (1, 1)
		p3 = matrix * (1, 0)
		self.writeln("initmatrix")
		self.writeln("%f %f moveto" % (p0[0], p0[1]))
		self.writeln("%f %f lineto" % (p1[0], p1[1]))
		self.writeln("%f %f lineto" % (p2[0], p2[1]))
		self.writeln("%f %f lineto" % (p3[0], p3[1]))
		self.writeln("closepath")
		self.writeln("clip")
		self.writeln("newpath")




	def pushClipPath(self):
		return
		self.writeln("gsave")
		viewportmatrix = self.getViewportMatrix()
		devicematrix = self.getDeviceMatrix()
		matrix = devicematrix * viewportmatrix
		print "clipping with: ", viewportmatrix.matrix
		p0 = matrix * (0, 0)
		p1 = matrix * (0, 1)
		p2 = matrix * (1, 1)
		p3 = matrix * (1, 0)
		m = viewportmatrix.matrix
		matrixstr = "%f %f %f %f %f %f" % (m[0][0], m[1][0], m[0][1], m[1][1], m[0][2], m[1][2])
		self.writeln("%% clipping with viewport matrix: %s " % (str(matrixstr)))
		self.writeln("%f %f moveto" % (p0[0], p0[1]))
		self.writeln("%f %f lineto" % (p1[0], p1[1]))
		self.writeln("%f %f lineto" % (p2[0], p2[1]))
		self.writeln("%f %f lineto" % (p3[0], p3[1]))
		self.writeln("closepath")
		self.writeln("clip")
		self.writeln("newpath")

	def pushReverseClipPath(self):
		return
		self.writeln("gsave")
		viewportmatrix = self.getViewportMatrix()
		devicematrix = self.getDeviceMatrix()
		matrix = devicematrix * viewportmatrix
		p0 = matrix * (0, 0)
		p1 = matrix * (0, 1)
		p2 = matrix * (1, 1)
		p3 = matrix * (1, 0)
		m = viewportmatrix.matrix
		matrixstr = "%f %f %f %f %f %f" % (m[0][0], m[1][0], m[0][1], m[1][1], m[0][2], m[1][2])
		self.writeln("%% clipping reverse with viewport matrix: %s " % (str(matrixstr)))
		self.writeln("clippath")
		self.writeln("reversepath")
		self.writeln("%f %f moveto" % (p0[0], p0[1]))
		self.writeln("%f %f lineto" % (p1[0], p1[1]))
		self.writeln("%f %f lineto" % (p2[0], p2[1]))
		self.writeln("%f %f lineto" % (p3[0], p3[1]))
		self.writeln("closepath")
		self.writeln("clip")
		self.writeln("newpath")


	def popClipPath(self):
		return
		self.writeln("grestore")

	def getLinestyles(self):
		return self.linestyles[:]

	def getFillstyles(self):
		return self.fillstyles[:]

	def getFonts(self):
		return self.fontMap.keys()
		
	def changeSize(self, size, units=None):
		norm = self.getUnitMatrix(units) * size
		self.res = self.getUnitMatrix("points").inverse() * norm
		self.setDimensions()
		#print "changeSize not implemented yet, will be ignored"

	def setBgColor(self, color):
		if (color != None) and (not hasattr(color, "r")):
			color = Kplot.Color.fromString(color)
		self.bgColor = color


	def setFgColor(self, color):
		if not hasattr(color, "r"):
			color = Kplot.Color.fromString(color)
		self.fgColor = color

"""
	def plotArc(self, p1, p2, fillstyle, linestyle, linewidth, color, startangle, anglelength, units):
		#x, y = self.transformation.transform(x, y)
		self.outputMatrix()
		#self.writeln("arc")
		#self.outputColor(color)
		#self.writeln("gsave")
		#self.outputFillstyle(fillstyle)
		#self.writeln("grestore")
		#self.outputLinestyle(linestyle)
		x1, y1 = p1
		x2, y2 = p2
		width = x2 - x1
		height = y2 - y1

		#code = ""
		#m = self.getTotalMatrix().matrix
		#code += "[%f %f %f %f %f %f] setmatrix\n" % (m[0][0], m[0][1], m[1][0], m[1][1], m[0][1], m[0][2])
		#self.file.write(code)

		x = x1 + width/2.0
		y = y1 + height/2.0
		radius = max(width, height) / 2
		m = Kplot.Matrix.scaleXY(width/radius, height/radius).matrix
		#self.writeln("[%f %f %f %f %f %f] concat" % (m[0][0], m[0][1], m[1][0], m[1][1], m[0][1], m[0][2]))
		#print x, y, radius

		self.writeln("%f %f moveto" % (x, y))
		self.writeln("%f %f %f %f %f arc" % (x, y, radius, startangle, anglelength))
		self.writeln("closepath")
		self.writeln("initmatrix")
		self.outputColor(color)
		self.outputFillstyle(fillstyle)
		#self.writeln("fill")

		#self.writeln("gsave")
		#self.
		#self.writeln("closepath")
		#self.outputFillstyle(fillstyle)
		#self.writeln("grestore")
		#self.outputLinestyle(linestyle)

		pass
"""
