import types
import Kplot
import qt
from qt import *
from Numeric import *
from panel import Panel
import sys, traceback
from pdb import *


class QtDevice(Kplot.DeviceAbstract):
	def __init__(self, widget, autoupdate=1, maxcolor=255):
		Kplot.DeviceAbstract.__init__(self)

		self.autoupdate = autoupdate
		self.maxcolor = maxcolor
		self.size = size
		self.widget = widget
		self.opened = 0

		self.open()


		self.width = self.widget.width()
		self.height = self.widget.height()
		self.clipAreaStack = []
		self.currClipArea = self.makeQtClipArea()
		self.clipAreaStack.append(self.currClipArea)

		self.linestyles = {	"normal":qt.QPen.SolidLine,
							"noline":qt.QPen.NoPen,
							"dashed":qt.QPen.DashLine,
							"dotted":qt.QPen.DotLine,
							"dotdash":qt.QPen.DashDotLine,
							"dashdotdot":qt.QPen.DashDotDotLine
							}
		self.fillstyles = {	"nofill":qt.QBrush.NoBrush,
							"solid":qt.QBrush.SolidPattern,
							"hatched":qt.QBrush.BDiagPattern,
							"crosshatched": qt.QBrush.DiagCrossPattern
							}

		self.fonts = {	"normal": qt.QFont("courier"),
						"normal2": qt.QFont("Times new Roman")
					}
		self.setClipping(1)
		metrics = QPaintDeviceMetrics(self.widget)
		widthmm, heightmm = metrics.widthMM(), metrics.heightMM()
		size = self.widget.size()
		width, height = size.width(), size.height()
		self.pixelwidth = widthmm/width
		self.pixelheight = heightmm/height


	def changeSize(self, size, units=None):
		norm = self.getUnitMatrix(units) * size
		width, height = self.widget.size().width(), self.widget.size().height()
		newdimension = Kplot.Matrix.scaleXY(width, height) * norm
		width, height = newdimension
		self.widget.setFixedSize(width, height)
		#aspect = float(height) / width
		#ppgplot.pgpap(width, aspect)
		#ppgplot.pgpage()
		
	def setDimensions(self):
		metrics = QPaintDeviceMetrics(self.widget)
		width, height = metrics.widthMM(), metrics.heightMM()
		mm = Kplot.Vector(width, height)
		self.dimensions["mm"] = mm
		self.dimensions["cm"] = mm.scale(0.1)
		self.dimensions["inch"] = mm.scale(1.0 / 25.4)
		self.dimensions["points"] = mm.scale(1.0 / 25.4 * 72)
		

	def getFillstyles(self):
		return self.fillstyles.keys()

	def getLinestyles(self):
		return self.linestyles.keys()

	def getFonts(self):
		return self.fonts.keys()

	def open(self):
		assert(not self.opened)
		self.opened = 1
		self.makePaintDevice()

	def getClipping(self):
		return self.clipping

	def close(self):
		print "close..."
		self.opened = 0
		self.destroyPaintDevice()

	def __del__(self):
		self.close()

	def paintEvent(self):
		if self.autoupdate:
			self.output()
		else:
			print "blit"
			qt.bitBlt(self.widget, 0, 0, self.paintDevice)

	def output(self):
		self.width = self.widget.width()
		self.height = self.widget.height()
		self.clipAreaStack = []
		self.currClipArea = self.makeQtClipArea()
		self.clipAreaStack.append(self.currClipArea)
		self.makePaintDevice()
		for child in self.children:
			child.plot(self)
		qt.bitBlt(self.widget, 0, 0, self.paintDevice)


	def makePaintDevice(self):
		width = self.widget.width()
		height = self.widget.height()
		self.paintDevice = qt.QPixmap(self.widget.size())

	def destroyPaintDevice(self):
		pass


	def getDeviceMatrix(self):
		return Kplot.Matrix.translate((0, self.height)) * Kplot.Matrix.scaleXY(self.width, -self.height)
		#return Kplot.Matrix.scaleXY(self.width, self.height)

	def getPanel(self, p1=(0.1,0.1), p2=(0.9, 0.9), parent=None):
		if parent == None:
			parent = self
		panel = Panel(p1=p1, p2=p2, parent=parent)
		if parent == self:
			self.children.append(panel)
		return panel

	def getQtLinestyle(self, linestyle):
		if linestyle not in self.linestyles:
			raise Kplot.CapsException, "Unknown linestyle: " +str(linestyle)
		else:
			return self.linestyles[linestyle]

	def getQtColor(self, color):
		if type(color) == types.StringType:
			c = Kplot.Color.fromString(color)
		else:
			c = color
		r, g, b = c.getRGB()
		return qt.QColor(r*255, g*255, b*255)

	def getQtFillstyle(self, fillstyle):
		if fillstyle not in self.fillstyles:
			raise Kplot.CapsException, "Unknown fillstyle: " +str(fillstyle)
		else:
			return self.fillstyles[fillstyle]

	def getQtFont(self, font):
		if font not in self.fonts:
			raise Kplot.CapsException, "Unknown font: " +str(font)
		else:
			return self.fonts[font]

	def setQtMatrix(self, painter):
		matrix = self.getTotalMatrix() * Kplot.Matrix.scaleXY(1.0/(self.width), 1.0/(self.height))
		#matrix = self.getTotalMatrix()
		m = matrix.matrix
		qtmatrix = qt.QWMatrix(m[0][0], m[1][0], m[0][1], m[1][1], m[0][2], m[1][2])
		#qtmatrix = qt.QWMatrix(m[0][0], m[1][0], m[0][1], m[1][1], m[0][2], m[1][2])
		painter.setWorldMatrix(qtmatrix)

	def setClippingArea(self, painter):
		if self.clipping:
			painter.setClipping(1)
			painter.setClipRegion(self.currClipArea)
		else:
			painter.setClipping(0)

	def plotPolyline(self, x, y, linestyle, linewidth, color, units):
		x, y = self.transformation.transform(x, y)
		x, y = Kplot.Matrix.scaleXY(self.width, self.height).mulXY(x, y)

		length = min(len(x), len(y))

		painter = qt.QPainter()
		painter.begin(self.paintDevice)

		qtlinestyle = self.getQtLinestyle(linestyle)
		qtcolor = self.getQtColor(color)
		pen = qt.QPen(qtcolor, linewidth, qtlinestyle)
		self.setQtMatrix(painter)
		self.setClippingArea(painter)
		painter.setPen(pen)
		pointarray = qt.QPointArray(length)
		for i in xrange(0, length):
			pointarray.setPoint(i, x[i], y[i])
		painter.drawPolyline(pointarray)
		painter.end()


	def plotPolygon(self, x, y, fillstyle, linestyle, linewidth, color, units):
		#return
		x, y = self.transformation.transform(x, y)
		x, y = Kplot.Matrix.scaleXY(self.width, self.height).mulXY(x, y)
		length = min(len(x), len(y))

		qtlinestyle = self.getQtLinestyle(linestyle)
		qtcolor = self.getQtColor(color)
		qtfillstyle = self.getQtFillstyle(fillstyle)

		pen = qt.QPen(qtcolor, linewidth, qtlinestyle)
		brush = qt.QBrush(qtcolor, qtfillstyle)

		#set_trace()
		#print "1"
		painter = qt.QPainter()
		painter.begin(self.paintDevice)
		self.setQtMatrix(painter)
		self.setClippingArea(painter)
		#matrix = qt.QWMatrix()

		painter.setPen(pen)
		painter.setBrush(brush)
		pointarray = qt.QPointArray(length)
		for i in xrange(0, length):
			pointarray.setPoint(i, x[i], y[i])
		painter.drawPolygon(pointarray)
		painter.end()



	def plotArc(self, p1, p2, fillstyle, linestyle, linewidth, color, startangle, anglelength, units):
		matrix = Kplot.Matrix.scaleXY(self.width, self.height)
		p1 = matrix * p1
		p2 = matrix * p2
		qtlinestyle = self.getQtLinestyle(linestyle)
		qtcolor = self.getQtColor(color)
		qtfillstyle = self.getQtFillstyle(fillstyle)

		pen = qt.QPen(qtcolor, linewidth, qtlinestyle)
		brush = qt.QBrush(qtcolor, qtfillstyle)

		painter = qt.QPainter()
		painter.begin(self.paintDevice)
		self.setQtMatrix(painter)
		self.setClippingArea(painter)
		painter.setPen(pen)
		painter.setBrush(brush)

		x, y = p1
		w, h = p2[0] - x, p2[1] - y
		painter.drawPie(x, y, w, h, startangle*16, anglelength*16)
		painter.end()

	def plotText(self, text, location, font, fontsize, weight, color, bgcolor, textangle, valign, halign, units):
		matrix = self.getTotalMatrix()#Kplot.Matrix.scaleXY(self.width, self.height)
		location = matrix * location

		#matrix = self.getTotalMatrix() * Kplot.Matrix.scaleXY(1.0/(self.width), 1.0/(self.height))
		#matrix = self.getTotalMatrix()
		#m = matrix.matrix
		#qtmatrix = qt.QWMatrix(m[0][0], m[1][0], m[0][1], m[1][1], m[0][2], m[1][2])
		#qtmatrix = qt.QWMatrix(m[0][0], m[1][0], m[0][1], m[1][1], m[0][2], m[1][2])
		#painter.setWorldMatrix(qtmatrix)
		#l


		qtfont = self.getQtFont(font)
		#print fontsize
		#print qtfont.family()
		qtfont.setPointSize(fontsize)
		#qtfont.setWeight(fontsize)
		qtcolor = self.getQtColor(color)

		pen = qt.QPen(qtcolor, fontsize)


		painter = qt.QPainter()
		painter.begin(self.paintDevice)
		self.setClippingArea(painter)

		haligns = {"left":qt.QPainter.AlignLeft,
					"right": qt.QPainter.AlignRight,
					"center": qt.QPainter.AlignHCenter }
		valigns = {"top": qt.QPainter.AlignTop,
					"bottom": qt.QPainter.AlignBottom,
					"center": qt.QPainter.AlignVCenter }
		matrix = qt.QWMatrix()
		x, y = location
		#print textangle
		#print "ta",textangle
		m = Kplot.Matrix(self.getViewportMatrix().matrix)
		m.matrix[0][2] = 0
		m.matrix[1][2] = 0
		textangle += (m * Kplot.Vector(1,0)).angle() * 180/pi
		#print "add", (m * Kplot.Vector(1,0)).angle() * 180/pi
		matrix.translate(x, y)
		matrix.rotate(-textangle)
		matrix.translate(-x, -y)
		painter.setWorldMatrix(matrix)
		#painter.setWorldMatrix(matrix)

		#self.setQtMatrix(painter)
		#painter.setWorldXForm(0)
		painter.setFont(qtfont)
		painter.setPen(pen)

		x, y = location
		flag = 0
		flag |= haligns[halign]
		flag |= valigns[valign]
		flag |= qt.QPainter.DontClip
		painter.drawText(x, y, 0, 0, flag, text)
		#painter.drawText(x, y, 1000, 1000, 0, text)
		painter.end()

	def plotImage(self, data, colormap, datamin, datamax, function, origin = (1,1)):
		matrix = self.getTotalMatrix()
		m = matrix.matrix
		qtmatrix = qt.QWMatrix(m[0][0], m[1][0], m[0][1], m[1][1], m[0][2], m[1][2])

		if datamin > datamax:
			datamin, datamax = datamax, datamin

		painter = qt.QPainter()
		painter.begin(self.paintDevice)
		painter.setWorldMatrix(qtmatrix)
		self.setClippingArea(painter)

		width = len(data[0])
		height = len(data)
		#flat = ravel(data)
		#origmin, origmax = min(flat), max(flat)
		#origwidth = origmax - origmin


		colorwidth = datamax-datamin
		data = clip(data, datamin, datamax)
		scale = float(self.maxcolor) / colorwidth
		data = (array(data) - datamin)* scale

		image = qt.QImage(width, height, 8)
		image.setNumColors(self.maxcolor+1)
		#print palwidth, palstart, origwidth, colorwidth
		#print origmin, origmax, datamin, datamax
		for i in xrange(self.maxcolor+1):
			index = float(i)/(self.maxcolor)# * palwidth + palstart
			index = function(index) # * index
			index = clip(index, 0.0, 1.0)
			#print index
			c = colormap.getColor(index)
			r = c.r * 255
			g = c.g * 255
			b = c.b * 255
			image.setColor(i, qt.qRgb(r, g, b))
		for x in xrange(width):
			for y in range(height):
				image.setPixel(x, y, data[y][x])
		painter.drawImage(qt.QPoint(0,0), image)
		self.setClippingArea(painter)

		painter.end()

	def makeQtClipArea(self):
		viewportmatrix = self.getViewportMatrix()
		devicematrix = self.getDeviceMatrix()
		matrix = devicematrix * viewportmatrix
		p0 = matrix * (0, 0)
		p1 = matrix * (1, 0)
		p2 = matrix * (1, 1)
		p3 = matrix * (0, 1)
		pointarray = qt.QPointArray(4)
		pointarray.setPoint(0, p0[0], p0[1])
		pointarray.setPoint(1, p1[0], p1[1])
		pointarray.setPoint(2, p2[0], p2[1])
		pointarray.setPoint(3, p3[0], p3[1])
		return qt.QRegion(pointarray)

	def pushClipArea(self):
		self.clipAreaStack.append(self.currClipArea)

	def popClipArea(self):
		self.currClipArea = self.clipAreaStack.pop()

	def subtractClipArea(self):
		newClipArea = self.makeQtClipArea()
		self.currClipArea = self.currClipArea.subtract(newClipArea)

	def intersectClipArea(self):
		newClipArea = self.makeQtClipArea()
		self.currClipArea = self.currClipArea.intersect(newClipArea)



