import Kplot

class MarkerGraph(object):
	"""A graph showing a marker at each x,y location.

	This class is almost the same as the Kplot.simple.Graph class.
	
	example:
		import Kplot
		x = range(6)
		y = [1,6,8,3,5,9]
		m = Kplot.simple.MarkerLine(x, y)
		m.box.drawgrid = False

	Attributes of MarkerGraph:
		device:      a Kplot.Device object
		panel:       a Kplot.Panel object
		markerline:  the graph itself, a Kplot.MarkerLine object
		box:         the grid and axes, a Kplot.Box object
		toplabel:
		leftlabel:
		bottomlabel:
		rightlabel: the four labels around the graph, bottomlabel has the text
		            "x", the leftlabel the text "y"

	example:
		Connect the markers:
		>>> m.markerline.connect = True
		
		Change the color of the markers:
		>>> m.markerline.markers.color = "red"
		
		Make the markers 10mm large:
		>>> m.markerline.markers.markersize = 10
		>>> m.markerline.markers.units = "mm"
		
		Change the markertype to a solidstar:
		>>> m.markerline.markers.markertype = Kplot.markertype.solidstar


	"""
	def __init__(self, x, y, markertype = Kplot.markertype.x, p1=None,p2=None):
		self.device = Kplot.createDevice()
		self.device.setAutoReplot(False)
		#self.x, self.y = x, y

		self.panel = self.device.createPanel()
		self.markerline = Kplot.MarkerLine(x,y, markertype)
		self.box = Kplot.Box()
		self.toplabel = Kplot.Label("", "top")
		self.leftlabel = Kplot.Label("y", "left", displacement=15)
		self.bottomlabel = Kplot.Label("x", "bottom", displacement=15)
		self.rightlabel = Kplot.Label("", "right")
		self.panel.add(self.box)
		self.panel.add(self.markerline)
		self.panel.add(self.toplabel, self.bottomlabel, self.leftlabel, self.rightlabel)
		self.setWorld(p1,p2)
		self.device.setAutoReplot(True)

	def __del__(self):
		self.device.close()

	def setWorld(self,p1,p2):
		if p1 and p2:
			self.panel.setWorld(p1,p2)
		else:
			x, y = self.markerline.x, self.markerline.y
			ymargin = 0.1 * (max(y)-min(y))
			xmargin = 0.1 * (max(x)-min(x))
			self.panel.setWorldXY(x, y,xmargin=xmargin,ymargin=ymargin)
		self.replot()

	def replot(self):
		self.device.output()

