chapter("Using Python (and Kplot)", 0)
text("""Before starting you might want to read the Python tutorial at
<a href="http://www.python.org/doc/current/tut/tut.html">
http://www.python.org/doc/current/tut/tut.html</a>.
""")
chapter("Using help", 1)
text("""This tutorial doesn't cover everything in Kplot. But Kplot itself is well
documented. The documentation is in Kplot itself, and can be
read by using the help function. This doesn't apply to Kplot alone, but to
everyhing in Python. The help function in Python shows the documentation.
""")
code("""python2.2
Python 2.2.1 (#1, Apr 11 2002, 15:27:15)
[GCC 2.95.3 20010315 (release)] on linux2
Type "help", "copyright", "credits" or "license" for more information.
>>> import Kplot
>>> help(Kplot)
>>> help(Kplot.simple.Graph)
""")
text("""The same documentation can be at
<a href="http://www.astro.rug.nl/~kplot/">
http://www.astro.rug.nl/~kplot/</a>. It's called the API
documentation.""")
chapter("Using Numeric", 1)
text("""Kplot uses a Python package called Numeric. This makes it easy to
make an array based on a function. The Numeric.arange(same as
Numeric.arrayrange) will be used throughout this document. Numeric.arange(5)
creates an arrays [0,1,2,3,4]. When given two arguments (start, stop),
Numeric.arange(1,6) it will return the arrays [1,2,3,4,5]. The last form,
in which it is given 3 arguments(start, stop, step), Numeric.arange(1,6,0.5)
returns a arrays with different step than 1
[1, 1.5, 2, 2.5, 3, 3.5, 4, 4.5, 5, 5.5]. New arrays can be created
by using operators like -,+,* and some functions like Numeric.sin, Numeric.cos.
""")
code("""python2.2
Python 2.2.1 (#1, Apr 11 2002, 15:27:15)
[GCC 2.95.3 20010315 (release)] on linux2
Type "help", "copyright", "credits" or "license" for more information.
>>> import Numeric
>>> x = Numeric.arange(10)
>>> x
array([0, 1, 2, 3, 4, 5, 6, 7, 8, 9])
>>> y = x * 2 + 10
>>> y
array([10, 12, 14, 16, 18, 20, 22, 24, 26, 28])""")
note("""Printing an expression in the Python console like 'x' will print it out.""")
note("""The Numeric homepage can be found at <a href="http://www.numpy.org/">http://www.numpy.org/</a>.""")

chapter("Using Classes", 1)
text("""Kplot is Object Oriented, and therefore uses classes. Classes are like
blueprints. You can creates several objects from a class. An object is also 
called an instance of a class.
Making an object involves calling the constructor, the classname with arguments""")
code(""">>> import Kplot
>>> color = Kplot.Color(1,0,0)
>>> color
<class Kplot.color.Color instance r=1.0000 g=0.0000 b=0.0000>
>>> help(Kplot.Color)
""")
text("""The arguments given to the constructor can be found in the help
documentation. Look for the __init__ method, this will be called when
creating an object. A preview of the help can be seen below.""")
code("""
Help on class Color in module Kplot.color:

class Color(__builtin__.object)
 |  Colors consist of values red, green and blue values between 0 and 1
 |
 |  3 ways to get a color
 |
 |  redcolor = Color(1,0,0)
 |  redcolor = Color.red
 |  redcolor = Color.fromString("red")
 |
 |  the Kplot.fromString method is the reason that in Kplot you may use a string
 |  instead of a Kplot.Color object to represent a color.
 |
 |  Methods defined here:
 |
 |  __eq__(self, color)
 |
 |  __init__(self, r, g, b)
 |      Constructor
 |
 |      Arguments:
 |      r,g,b -- A value between 0 and 1
 |
 |  __ne__(self, color)
 |
 |  __repr__(self)
 |
 |  __str__(self)
 |
 |  distance(self, color)
 |      Returns the distance between the 2 colors seen from a r,b,g color cube
 |
 |  getRGB(self)
 |      Return a (r,g,b) tuple
 |
 |  ----------------------------------------------------------------------

""")

text("""An object also has methods, they are like function, but an
'object function' is called a method. In the API documentation you see
the first argument to Color.getRGB is 'self', this is an implicit
argument, and is filled in by Python.
""")
code(""">>> color.getRGB()
(1.0, 0.0, 0.0)
>>>""")
