import os

chapter("Simple interface", 0)
text("""The classes described in this chapter provide a simple interface
to make straightforward plots in just a few lines. It is meant to make simple plots, for more
complex plots you have to learn more about Kplot in the chapter 'Normal interface'.
""")
chapter("Graph",1)
text("""The simple interface consists of a couple of classes. The graph
class provides a simple interface to draw a x,y plot, a graph.
The first thing to do is make an Numeric x and y array. We'll use the simple function
<i>y = x ** 2</i>(x ** 2 means x to the power of 2). Start a python2.2 console by
typing 'python2.2'. The first thing to do is to import the Numeric package, and make an x array.
Note that Numeric.arange is short for Numeric.arrayrange.""")
note("""When typing an expression in the
python console, the expression will be printed when the result of the expression is
not None. Thus typing 'x'<enter> will print the x array on your console.""")
code("""
@~>python2.2
Python 2.2.1 (#1, Apr 11 2002, 15:27:15)
[GCC 2.95.3 20010315 (release)] on linux2
Type "help", "copyright", "credits" or "license" for more information.
>>> import Numeric
>>> x = Numeric.arange(11)
>>> x
array([ 0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10])
>>> y = x ** 2
>>> y
array([  0,   1,   4,   9,  16,  25,  36,  49,  64,  81, 100])
""")
text("""Now it's time to make a graph. Import the Kplot package. You can make a
Graph object by calling the Graph constructor with the arrays x and a y you just constructed.""")
code("""
>>> import Kplot
>>> g = Kplot.simple.Graph(x, y)
""")
text("""A graph is shown(on the default device), and it can be changed by accessing the properties
of the Graph members. You can explore Graph's member by simply typing 'g.line'
When a properties is changed, Kplot will detect it and will replot everything.
""")
note("""How to change the default device, can be found in a later chapter.""")
note("""Note that only assignment to one of the properties will be detected,
changing the x array will not trigger this replot. Calling g.replot() will force
to replot everything.""")
code("""
>>> g.line
Polyline attributes:
        matrix-Kplot.matrix.Matrix&lt;[[ 1.  0.  0.]
 [ 0.  1.  0.]
 [ 0.  0.  1.]]&gt;
        color-red
        y-[  0   1   4   9  16  25  36  49  64  81 100]
        units-None
        x-[ 0  1  2  3  4  5  6  7  8  9 10]
        linewidth-0
        linestyle-normal
>>> g.line.color = "red"
>>> g.box.yinterval = 20""")
note("""More details about the properties of this object can be obtained by reading
the API specifications. Starting at Kplot.simple.Graph help(using the help function), 
you will learn that it has several member, line being a Kplot.Polyline(or Kplot.plotobjects.Polyline).
Calling help on this class or browsing the API documentation online will give you more information
about this class.
.""")
code(""">>> help(Kplot.simple.Graph)""")
code("""Help on class Graph in module Kplot.simple.graph:

class Graph(__builtin__.object)
 |  Plots a simple x/y graph
 |
 |  example:
 |          import Kplot
 |          g = Kplot.simple.Graph(x=[0,1,2,3], y=[3,4,7,2])
 |
 |  properties of PlotObjects can be changed, and the plot will repaint itself.
 |
 |  Attributes of Graph:
 |          device:      a Kplot.Device object
 |          panel:       a Kplot.Panel object
 |          line:        the graph itself, a Kplot.Polyline object
 |          box:         the grid and axes, a Kplot.Box object
 |          toplabel:
 |          leftlabel:
 |          bottomlabel:
 |          rightlabel: the four labels around the graph, bottomlabel has the text
 |                      "x", the leftlabel the text "y"
 |
 |  example:
 |          g.toplabel.text.text = "f(x) = x ** 2"
 |          g.toplabel.text.color = "red"
 |          g.box.xinterval = 1
 |          g.box.drawgrid = False
 |
 |  g.toplabel.text.text may look strange, but g.toplabel.text is a Kplot.Text
 |  object, where text is one of it's properties.
 |
 |  To find information about a PlotObject, use <object>.printInfo()
 |  or use the help function. The help function will give a description of the
 |  class.
 |
 |  example:
 |          g.box.printInfo()
 |          help(Kplot.Box)
 |
 |  Methods defined here:
 |
 |  __del__(self)
 |
 |  __init__(self, x, y)
 |
 |  replot(self)
 |
 |  setWorld(self)
""")

os.environ["KPLOT_FILENAME"] = "graph"
os.system("cd images; python graph.py")
image("images/graph.png")

chapter("MarkerGraph",1)
text("""MarkerGraph is almost the same as the Graph class, but has markers
at every x,y location. The constructor has an optional argument, the markertype,
it's default being Kplot.markertype.x, a cross shape.""")
note("""See the Kplot.markertype module in the API reference for more markertypes.""")
code("""
>>> m = Kplot.simple.MarkerGraph(x, y, Kplot.markertype.solidstar)
>>> m.markerline.markers.color = "red"
>>> m.markerline.polyline.color = "blue"
""")

os.environ["KPLOT_FILENAME"] = "scatter"
os.system("cd images; python scatter.py")
image("images/scatter.png")

note("""For more detail check Kplot.simple.MarkerGraph, and since it's member scatter
is of class Kplot.plotobjects.MarkerLine you might want to check it too.""")

code("""
>>> help(Kplot.simple.MarkerGraph)
""")
code("""
Help on class MarkerGraph in module Kplot.simple.markergraph:

class MarkerGraph(__builtin__.object)
 |  A graph showing a marker at each x,y location.
 |
 |  This class is almost the same as the Kplot.simple.Graph class.
 |
 |  example:
 |          import Kplot
 |          x = range(6)
 |          y = [1,6,8,3,5,9]
 |          m = Kplot.simple.MarkerLine(x, y)
 |          m.box.drawgrid = False
 |
 |  Attributes of MarkerGraph:
 |          device:      a Kplot.Device object
 |          panel:       a Kplot.Panel object
 |          markerline:  the graph itself, a Kplot.MarkerLine object
 |          box:         the grid and axes, a Kplot.Box object
 |          toplabel:
 |          leftlabel:
 |          bottomlabel:
 |          rightlabel: the four labels around the graph, bottomlabel has the text
 |                      "x", the leftlabel the text "y"
 |
 |  example:
 |          Connect the markers:
 |          >>> m.markerline.connect = True
 |
 |          Change the color of the markers:
 |          >>> m.markerline.markers.color = "red"
 |
 |          Make the markers 10mm large:
 |          >>> m.markerline.markers.markersize = 10
 |          >>> m.markerline.markers.units = "mm"
 |
 |          Change the markertype to a solidstar:
 |          >>> m.markerline.markers.markertype = Kplot.markertype.solidstar
 |
 |  Methods defined here:
 |
 |  __del__(self)
 |
 |  __init__(self, x, y, markertype=<Kplot.points.point.Point instance>)
 |
 |  replot(self)
 |
 |  setWorld(self)
 |
 |  ----------------------------------------------------------------------
""")
text("""As you can read in the documentation, Kplot.simple.MarkerGraph has a attribute
called markerline which is a Kplot.MarkerLine class. You can use the help method on this
class to get more information about it.""")
code("""
>>> help(Kplot.MarkerLine)
""")

code("""
......
 |  connect = attribute
 |      If True, a Polyline will be plotted through the markers
 |      default: 0
 |      allowed types: any type
 |
 |  markers = attribute
 |      The Markers object that will be plotted
 |      default: no default
 |      allowed types: <class 'Kplot.plotobjects.nonbasic.Markers'>
 |
......
""")

text("""As your can see in this part of the API documentation, there is an
attribute called connect. And if this is set to True, the markers will be 
connected. If m is this the MarkerGraph object, the code below will connect
the markers.""")
code("""
>>> m.markerline.connect = True
""")

os.environ["KPLOT_FILENAME"] = "markergraph2"
os.system("cd images; python markergraph2.py")
image("images/markergraph2.png")

chapter("Histogram",1)
text("""The Histogram class displays binned data. The documentation in 
Kplot.simple.Histogram should help give you enough information.""")
code(""">>> help(Kplot.simple.Histogram)

Help on class Histogram in module Kplot.simple.histogram:

class Histogram(__builtin__.object)
 |  Plots a simple histogram using binned data
 |
 |  example:
 |          import Kplot
 |          data = [5,2,9,4,1,4,6,2]
 |          g = Kplot.simple.Histogram(data)
 |
 |  Attributes of Graph:
 |    device:      a Kplot.Device object
 |    panel:       a Kplot.Panel object
 |    histogram:   the histogram itself, a Kplot.Histogram object
 |    box:         the grid and axes, a Kplot.Box object
 |    toplabel:
..................
""")

code("""
>>> import Kplot
>>> data = [5,2,9,4,1,4,6,2]
>>> g = Kplot.simple.Histogram(data)
>>> g.histogram
<Kplot.plotobjects.histogram.Histogram object at 0x82afed4>
Histogram attributes:
        matrix-Kplot.matrix.Matrix<[[ 1.  0.  0.]
 [ 0.  1.  0.]
 [ 0.  0.  1.]]>
        color-foreground
        fillstyle-crosshatched
        units-None
        bindata-[5, 2, 9, 4, 1, 4, 6, 2]
        polyline-0
        linewidth-0
        linestyle-normal

>>> g.histogram.fillstyle = "crosshatched"
""")

os.environ["KPLOT_FILENAME"] = "histogram"
os.system("cd images; python histogram.py")
image("images/histogram.png")
