chapter("Devices", 0)
text("""This chapter covers the Devices available in Kplot. 
At first a list of devices and a short explaination.""")
code("""
Kplot.pgplot.WindowDevice
	- X11 window using PGPLOT
Kplot.pgplot.GifDevice
	- Gif file, limited to 256 colors.
Kplot.pgplot.PsLandscapeDevice
	- PostScript file, landscape format
Kplot.pgplot.PsPortraitDevice
	- PostScript file, portrait format
Kplot.pgplot.NullDevice
	- No output, for testing etc.

Kplot.ps.WindowDevice
	- window using Ghostscript
Kplot.ps.PngDevice
	- PNG file using Ghostscript
Kplot.ps.FileDevice
	- PostScript file, using no library or program, portrait format
Kplot.ps.NullDevice
	- No output, for testing etc.
""")
chapter("PGPLOT", 1)
text("""The following Devices are implemented using the PGPLOT subroutine library.
The PGPLOT  library has numerous output options, depending on your PGPLOT version.
The Kplot.pgplot.WindowDevice outputs on a X11 window. The Kplot.pgplot.GifDevice to a gif
file. Kplot.pgplot.PsLandscapeDevice and Kplot.pgplot.PsPortraitDevice to a PostScript file.
""")

code("""
$ setenv KPLOT_DEVICE Kplot.pgplot.PsPortraitDevice
$ python yourscript.py
""")

text("""Another option, to support the whole list of devices listed at
<a href="http://www.astro.caltech.edu/~tjp/pgplot/devices.html">
http://www.astro.caltech.edu/~tjp/pgplot/devices.html</a>,
is to use the Kplot.pgplot.Device.
It takes an optional devicename argument, when not given,
it uses the KPLOT_PGPLOT_DEVICE environmental variable.
The Kplot.createDevice doesn't give the devicename argument,
so you will have to set the KPLOT_PGPLOT_DEVICE environmental
variable when KPLOT_DEVICE it set to Kplot.pgplot.Device.
""")
code("""
$ setenv KPLOT_DEVICE Kplot.pgplot.Device
$ setenv KPLOT_PGPLOT_DEVICE "test.gif/GIF"
$ python -c "import Kplot; Kplot.Text.example()"
""")

text("""The good thing about PGPLOT is that the output on each device looks pretty much the same.
It uses it's own Hershey font, made up of lines. They don't look very good, but
gives the same output on PostScript as on your window, and doesn't need an external
font.""")



chapter("Postscript/Ghostscript", 1)
text("""The Kplot.ps.FileDevice gives plain PostScript output to a file, 
and doesn't need any external libary or program.
The Kplot.ps.WindowDevice and Kplot.ps.PngDevice need
the Ghostscript program. The first will output to a window. The latter outputs
to a PNG file. Modern webbrowsers can all display this format, and it is free
of any patents.
""")

text("""To test if Ghostscript is installed, type 'gs'<enter>. If Ghostscript
is installed, but doesn't execute with the gs commando like in Windows, the
GS_BINARY environmental variable can be set to the executable filename.
""")

code("""
C:\>set GS_BINARY=F:\\Progs\\gs\\gs7.04\\bin\\gswin32c.exe
""")

text("""The environmental variable KPLOT_GS_CMDLINE can be set to add optional commands
to the Ghostscript interpreter. If set to "-dTextAlphaBits=4", text will
be anti-aliased(smoother). This only has effect on raster output, like on a
window, or a PNG file. More options can be found in the Ghostscript manual at
<a href="http://www.ghostscript.com/">http://www.ghostscript.com/</a>.
""")

code("""
$ setenv KPLOT_GS_CMDLIKE "-dTextAlphaBits=4"

The -dGraphicsAlphaBits=4 options also anti-aliases lines etc.
$ setenv KPLOT_GS_CMDLIKE "-dTextAlphaBits=4 -dGraphicsAlphaBits=4"
""")
