class Document:
	def __init__(self):
		self.chapters = []

	def addChapter(self, chapter):
		self.chapters.append(chapter)

	def generate(self, output):
		count = 1
		for chapter in self.chapters:
			chapter.setChapterNr(str(count))
			count += 1
		for chapter in self.chapters:
			chapter.generate(output)

	def resolveChapter(self, title):
		res = None
		for chapter in self.chapters:
			res = chapter.resolveChapter(title)
			if res != None:
				return res
		raise Exception, "unresolved chapter " +str(title)



class Chapter:
	def __init__(self, title, document, depth):
		self.title = title
		self.document = document
		self.nr = None
		self.children = []
		self.paragraphs = []
		self.depth = depth

	def setChapterNr(self, nr):
		#print nr
		self.nr = nr
		count = 1
		for child in self.children:
			child.setChapterNr(self.nr +"." + str(count))
			count += 1

	def addChapter(self, chapter):
		self.children.append(chapter)

	def addParagraph(self, para):
		self.paragraphs.append(para)

	def resolveChapter(self, title):
		if title == self.title:
			return self
		else:
			res = None
			for child in self.children:
				res = child.resolveChapter(title)
				if res != None:
					return res

	def __str__(self):
		return self.nr +" " +self.title

	def generate(self, output):
		#print str(self)
		output.chapter(str(self), self.depth)
		for para in self.paragraphs:
			para.generate(self.document, output)
		for child in self.children:
			child.generate(output)

class Paragraph:
	def generate(self, document, output):
		pass
		
	def parse(self, document, text):
		pass

class TextParagraph(Paragraph):
	def __init__(self, txt):
		self.txt = txt
	def generate(self, document, output):
		output.addText(self.txt)

class CodeParagraph(Paragraph):
	def __init__(self, txt):
		self.txt = txt
	def generate(self, document, output):
		output.addCode(self.txt)

class CommentParagraph(Paragraph):
	def __init__(self, txt):
		self.txt = txt
	def generate(self, document, output):
		output.addNote(self.txt)

class ImageParagraph:
	def __init__(self, url, width=None, height=None):
		self.url = url
		self.width = width
		self.height = height
	def generate(self, document, output):
		output.addImage(self.url, self.width, self.height)









