#!/usr/bin/env python2.2
from Kplot import *
import unittest
import Kplot

class DummyObject:
	def plot(self, dev):
		pass
class DummyObject(Kplot.plotobject.PlotObject):
	def __init__(self):
		Kplot.plotobject.PlotObject.__init__(self, {})
	def plot(self, dev):
		pass

class PanelTest(unittest.TestCase):
	def setUp(self):
		self.dev = pgplot.Device("/NULL")
		self.panel = self.dev.createPanel() #Panel((0,0), (1,1))

	def tearDown(self):
		self.dev.close()

	def testAddAndRemove(self):
		o1 = DummyObject()
		self.panel.add(o1)
		l = [DummyObject(), DummyObject()]
		self.panel.add(l)
		self.panel.remove(l)
		self.assert_(len(self.panel.objects) == 1)
		self.assert_(self.panel.objects[0], o1)

		self.panel.add([DummyObject(), [DummyObject(), DummyObject()]])
		self.panel.add((DummyObject(), DummyObject()))
		self.panel.add(DummyObject(), (DummyObject(), DummyObject()), DummyObject())
		for object in self.panel.objects:
			self.assert_(object.__class__, DummyObject)

	def testWorld(self):
		self.panel.setWorld(p1=(-10, -10), p2=(50,60))
		self.assertEquals(self.panel.worldmatrix * (-10, -10), Vector(0, 0))
		self.assertEquals(self.panel.worldmatrix * (50, 60), Vector(1, 1))
		p1, p2 = self.panel.getWorld()
		self.assertEquals(p1, Vector(-10, -10))
		self.assertEquals(p2, Vector(50, 60))

	def testViewport(self):
		self.panel.setViewport((0.1, 0.2), (0.8, 0.9))
		(x1, y1), (x2, y2) = self.panel.getViewport()
		self.assert_(abs(0.1 - x1) < 1e-10)
		self.assert_(abs(0.2 - y1) < 1e-10)
		self.assert_(abs(0.8 - x2) < 1e-10)
		self.assert_(abs(0.9 - y2) < 1e-10)
		self.assertEquals(self.panel.viewportmatrix * Vector(0, 0), Vector(0.1, 0.2))
		self.assertEquals(self.panel.viewportmatrix * Vector(1, 1), Vector(0.8, 0.9))

	def testMosiac(self):
		panels = self.panel.mosaic(2,3)
		self.assert_(len(panels) == 2)
		self.assert_(len(panels[0]) == 3)

	def testPlot(self):
		o1 = DummyObject()
		self.panel.add(o1)
		panels = self.panel.mosaic(1,2)
		self.panel.plot(self.dev)

	def testGetPanel(self):
		panel = self.panel.createPanel()
		self.assertEquals(panel.parent, self.panel)

		panels = self.panel.mosaic(1,2)
		for panel in panels:
			self.assertEquals(panel.parent, self.panel)

	def testPrint(self):
		self.panel.add(DummyObject())
		self.panel.printObjects()
		self.panel.printMembers()

	def testSplit(self):
		p1, p2 = self.panel.split(0.2, "vertical")
		self.assertEquals(p1.parent, self.panel)
		self.assertEquals(p2.parent, self.panel)
		self.panel.split(0.2, "horizontal")
		self.assertEquals(p1.parent, self.panel)
		self.assertEquals(p2.parent, self.panel)

	def testCopy(self):
		self.panel.background.color = "green"
		self.panel.background.fillstyle = "hatched"
		self.panel.background.linestyle = "dotted"
		panelcopy = self.panel.copy()
		self.assertEquals(panelcopy.background.color, "green")
		self.assertEquals(panelcopy.background.fillstyle, "hatched")
		self.assertEquals(panelcopy.background.linestyle, "dotted")

	def testOverlay(self):
		self.panel.background.color = "green"
		self.panel.background.fillstyle = "hatched"
		self.panel.background.linestyle = "dotted"
		panelcopy = self.panel.overlay(p1=(0,0), p2=(1,1))
		self.assertEquals(panelcopy.background.color, "green")
		self.assertEquals(panelcopy.background.fillstyle, "nofill")
		self.assertEquals(panelcopy.background.linestyle, "dotted")

	def testClipping(self):
		self.assertEquals(self.panel.getClipping(), True)
		self.panel.setClipping(False)
		self.assertEquals(self.panel.getClipping(), False)


if __name__ == '__main__':
	unittest.main()
	#t = PanelTest()
	#t.setUp()
	#t.testAddAndRemove()
